==> DNaseR_1.0.0.tar.gz: DNaseR/R/footprints.R <==

footprints <- function(bam, chrN, chrL, p=1e-9, width=c(6,40), N=5e6, correction="BH")
{

  NT = 1
  LL <- length(chrL)
  listLen <- 0
  SampleCovPlus <- list()
  SampleCovMinus <- list()

  for (j in 1:NT){

      for (i in 1:LL){
              listLen <-  listLen + 1
              which <- GRanges(seqnames = chrN[i], ranges = IRanges(c(1), c(chrL[i])))

              what <- c("rname", "strand", "pos", "qwidth")

              param <- ScanBamParam(which = which, what = what)
              SampleBam <- scanBam(file=bam[j], param=param)
              sapply(SampleBam[[1]], class);


              SamplelstPlus <- lapply(names(SampleBam[[1]]), function(elt) { do.call(c, unname(lapply(SampleBam, "[[", elt)))})
              names(SamplelstPlus) <- names(SampleBam[[1]])
              SampledfPlus <- do.call("DataFrame", SamplelstPlus)

              SampledfPlus$pos[which(SampledfPlus$strand==1)] <- SampledfPlus$pos[which(SampledfPlus$strand==1)] - 1

              SampledfPlus$qwidth <- 1

              SampledfPlus <- SampledfPlus[which(SampledfPlus$strand==1),]
              SampleCovPlus[[listLen]]  <- coverage(IRanges(SampledfPlus[["pos"]], width = SampledfPlus[["qwidth"]]))
              if (length(SampleCovPlus[[listLen]])   < chrL[i] ) { SampleCovPlus[[listLen]]   <- c(SampleCovPlus[[listLen]],rep(0,chrL[i] - length(SampleCovPlus[[listLen]]) ) )  }
              if (length(SampleCovPlus[[listLen]])   > chrL[i] ) { SampleCovPlus[[listLen]]   <-  SampleCovPlus[[listLen]][1:chrL[i]]     }


              rm(SampledfPlus,SamplelstPlus)
              SamplelstMinus <- lapply(names(SampleBam[[1]]), function(elt) { do.call(c, unname(lapply(SampleBam, "[[", elt)))})
              names(SamplelstMinus) <- names(SampleBam[[1]])
              SampledfMinus  <- do.call("DataFrame", SamplelstMinus )

              SampledfMinus$pos[which(SampledfMinus$strand==2)] <- SampledfMinus$pos[which(SampledfMinus$strand==2)] - 1 + SampledfMinus$qwidth[which(SampledfMinus$strand==2)]

              SampledfMinus$qwidth <- 1

              SampledfMinus <- SampledfMinus[which(SampledfMinus$strand==2),]
              SampleCovMinus[[listLen]]  <- coverage(IRanges(SampledfMinus[["pos"]], width = SampledfMinus[["qwidth"]]))
              if (length(SampleCovMinus[[listLen]])   < chrL[i] ) { SampleCovMinus[[listLen]]   <- c(SampleCovMinus[[listLen]],rep(0,chrL[i] - length(SampleCovMinus[[listLen]]) ) )  }
              if (length(SampleCovMinus[[listLen]])   > chrL[i] ) { SampleCovMinus[[listLen]]   <-  SampleCovMinus[[listLen]][1:chrL[i]]     }

              rm(SampleBam,SamplelstMinus,SampledfMinus,param,what,which)

      }

  }

 rm(i,j)


 listLen <- 0
 meanCovPlus <- c()
 meanCovMinus <- c()
 lambda1 <- c()
 lambda2 <- c()

 for (j in 1:NT){

      for (i in 1:LL){
              listLen <-  listLen + 1
              meanCovPlus[listLen]  <- mean(SampleCovPlus[[listLen]])
              meanCovMinus[listLen] <- mean(SampleCovMinus[[listLen]])
      }

      lambda1_ini <- sum((chrL/sum(chrL))  * meanCovPlus[(((j-1)*LL) + 1):(j*LL)])
      lambda2_ini <- sum((chrL/sum(chrL))  * meanCovMinus[(((j-1)*LL) + 1):(j*LL)])

      lambda1[j] <- max(lambda1_ini,lambda2_ini)
      lambda2[j] <- min(lambda1_ini,lambda2_ini)

      rm(lambda1_ini, lambda2_ini)

   }

 rm(i,j)


  listLen <-0
  width[1] <- width[1] -1
  width[2] <- width[2] +1


  for (w in 1:NT){
       filteredScoreFinalChr  <- data.frame()

      for (i in 1:LL){
          filteredScoreFinal     <- data.frame()
          listLen <-  listLen + 1
          mL <- max(length(SampleCovPlus[[listLen]]), length(SampleCovMinus[[listLen]]))
          frames <- floor(mL / N) +1
          for (m in 1:frames){

              if (m != frames){
                 lFrame <-  (((m-1)*N)+1):(m*N)
              }
               if (m == frames){
                  lFrame<- (((m-1)*N)+1):mL
              }
              sample_forward <- as.integer(SampleCovPlus[[listLen]] [  lFrame ])
              sample_reverse <- as.integer(SampleCovMinus[[listLen]][  lFrame ])


              k <- as.integer(sample_forward - sample_reverse)

              rm(sample_forward,sample_reverse)


              score <- c()
              iPlus  <- which(k>=0)
              iMinus <- which(k<0)
              score[iPlus]  <- pskellam(q=k[iPlus],    lambda1= lambda1[w], lambda2 = lambda2[w], lower.tail = FALSE, log.p = FALSE)
              score[iMinus] <- pskellam(q=k[iMinus]-1, lambda1= lambda1[w], lambda2 = lambda2[w], lower.tail = TRUE, log.p = FALSE)
              positions  <- which(score <= p)


              maxD = width[2]
              minD = width[1]

              filteredScore <- data.frame()
              idx=0

               if (length(positions)>1){
               
                  for (j in 2:length(positions)){
                  
                      A <- positions[j-1]
	                    B <- positions[j]
	                    C <- sign(k[positions[j-1]])
	                    D <- sign(k[positions[j]])

                      if (C != 0  && D != 0 ){
	                       if (B< A+maxD && B> A+minD  && (C == 1  && D== -1)  ){
		                        idx=idx+1
		                        filteredScore[idx,1] <- chrN[i]
		                        filteredScore[idx,2] <- lFrame[positions[j-1]]
		                        filteredScore[idx,3] <- lFrame[positions[j]]
		                        filteredScore[idx,4] <- lFrame[positions[j]]-lFrame[positions[j-1]]
		                        filteredScore[idx,5] <- signif(score[positions[j-1]], digits=30)
		                        filteredScore[idx,6] <- signif(score[positions[j]] , digits=30)
				                    pvalVector <- c(score[positions[j-1]] , score[positions[j]])
				                    filteredScore[idx,7] <- sum(pvalVector)
			                      filteredScore[idx,8] <- -1*log10(filteredScore[idx,7])

			                   }
		                  }
                  }
                }
                
              if ( length(filteredScoreFinal) > 0 &&  length(filteredScore) > 0) { filteredScoreFinal <- rbind(filteredScoreFinal,filteredScore)}
              if ( length(filteredScoreFinal) ==0  &&  length(filteredScore) > 0) { filteredScoreFinal <- filteredScore  }
                   


          }  #end frame

             if (length(filteredScoreFinalChr) > 0 && length(filteredScoreFinal) > 0 ) {  filteredScoreFinalChr <- rbind(filteredScoreFinalChr, filteredScoreFinal )  }
             if (length(filteredScoreFinalChr) ==0 && length(filteredScoreFinal) > 0 ) {  filteredScoreFinalChr <- filteredScoreFinal   }

     }  #end chromosome


  colnames(filteredScoreFinalChr) <- c("chr","start","end","length","pval.forward","pval.reverse", "pval.footprint.event","log10.pval.footprint.event")

}   #end sample




if (correction != "none") {

        filteredScoreFinalChr$pval.forward <- p.adjust(filteredScoreFinalChr$pval.forward, method=correction)
        filteredScoreFinalChr$pval.reverse <- p.adjust(filteredScoreFinalChr$pval.reverse, method=correction)
        for (i in 1:length(filteredScoreFinalChr$pval.reverse) ){
           filteredScoreFinalChr$pval.footprint.event[i]          <-  filteredScoreFinalChr$pval.forward[i] + filteredScoreFinalChr$pval.reverse[i]
           filteredScoreFinalChr$log10.pval.footprint.event[i]    <-   -1*log10(filteredScoreFinalChr$pval.footprint.event[i])
        }
        
}

   filteredScoreFinalChr  <-  filteredScoreFinalChr[ which(filteredScoreFinalChr$pval.footprint.event <= p  ) , ]
   
##   #create a GRanges structure
##   repl <- GRanges(seqnames=filteredScoreFinalChr$chr,ranges=IRanges(filteredScoreFinalChr$start, end=filteredScoreFinalChr$end), score=filteredScoreFinalChr$log10.pval.footprint.event)
##   seqlengths(repl)   <- chrL


##return(list( footprints = filteredScoreFinalChr , footprint.events = repl  ))
  return(list( footprint.events = filteredScoreFinalChr  ))


}

==> DNaseR_1.4.0.tar.gz: DNaseR/R/footprints.R <==

footprints <- function(bam, chrN, chrL, p=1e-9, width=c(6,40), N=5e6, correction="BH")
{

  NT = 1
  LL <- length(chrL)
  listLen <- 0
  SampleCovPlus <- list()
  SampleCovMinus <- list()

  for (j in 1:NT){

      for (i in 1:LL){
              listLen <-  listLen + 1
              which <- GRanges(seqnames = chrN[i], ranges = IRanges(c(1), c(chrL[i])))

              what <- c("rname", "strand", "pos", "qwidth")

              param <- ScanBamParam(which = which, what = what)
              SampleBam <- scanBam(file=bam[j], param=param)
              sapply(SampleBam[[1]], class);


              SamplelstPlus <- lapply(names(SampleBam[[1]]), function(elt) { do.call(c, unname(lapply(SampleBam, "[[", elt)))})
              names(SamplelstPlus) <- names(SampleBam[[1]])
              SampledfPlus <- do.call("DataFrame", SamplelstPlus)

              SampledfPlus$pos[which(SampledfPlus$strand==1)] <- SampledfPlus$pos[which(SampledfPlus$strand==1)] - 1

              SampledfPlus$qwidth <- 1

              SampledfPlus <- SampledfPlus[which(SampledfPlus$strand==1),]
              SampleCovPlus[[listLen]]  <- coverage(IRanges(SampledfPlus[["pos"]], width = SampledfPlus[["qwidth"]]))
              if (length(SampleCovPlus[[listLen]])   < chrL[i] ) { SampleCovPlus[[listLen]]   <- c(SampleCovPlus[[listLen]],rep(0,chrL[i] - length(SampleCovPlus[[listLen]]) ) )  }
              if (length(SampleCovPlus[[listLen]])   > chrL[i] ) { SampleCovPlus[[listLen]]   <-  SampleCovPlus[[listLen]][1:chrL[i]]     }


              rm(SampledfPlus,SamplelstPlus)
              SamplelstMinus <- lapply(names(SampleBam[[1]]), function(elt) { do.call(c, unname(lapply(SampleBam, "[[", elt)))})
              names(SamplelstMinus) <- names(SampleBam[[1]])
              SampledfMinus  <- do.call("DataFrame", SamplelstMinus )

              SampledfMinus$pos[which(SampledfMinus$strand==2)] <- SampledfMinus$pos[which(SampledfMinus$strand==2)] - 1 + SampledfMinus$qwidth[which(SampledfMinus$strand==2)]

              SampledfMinus$qwidth <- 1

              SampledfMinus <- SampledfMinus[which(SampledfMinus$strand==2),]
              SampleCovMinus[[listLen]]  <- coverage(IRanges(SampledfMinus[["pos"]], width = SampledfMinus[["qwidth"]]))
              if (length(SampleCovMinus[[listLen]])   < chrL[i] ) { SampleCovMinus[[listLen]]   <- c(SampleCovMinus[[listLen]],rep(0,chrL[i] - length(SampleCovMinus[[listLen]]) ) )  }
              if (length(SampleCovMinus[[listLen]])   > chrL[i] ) { SampleCovMinus[[listLen]]   <-  SampleCovMinus[[listLen]][1:chrL[i]]     }

              rm(SampleBam,SamplelstMinus,SampledfMinus,param,what,which)

      }

  }

 rm(i,j)


 listLen <- 0
 meanCovPlus <- c()
 meanCovMinus <- c()
 lambda1 <- c()
 lambda2 <- c()

 for (j in 1:NT){

      for (i in 1:LL){
              listLen <-  listLen + 1
              meanCovPlus[listLen]  <- mean(SampleCovPlus[[listLen]])
              meanCovMinus[listLen] <- mean(SampleCovMinus[[listLen]])
      }

      lambda1_ini <- sum((chrL/sum(chrL))  * meanCovPlus[(((j-1)*LL) + 1):(j*LL)])
      lambda2_ini <- sum((chrL/sum(chrL))  * meanCovMinus[(((j-1)*LL) + 1):(j*LL)])

      lambda1[j] <- max(lambda1_ini,lambda2_ini)
      lambda2[j] <- min(lambda1_ini,lambda2_ini)

      rm(lambda1_ini, lambda2_ini)

   }

 rm(i,j)


  listLen <-0
  width[1] <- width[1] -1
  width[2] <- width[2] +1


  for (w in 1:NT){
       filteredScoreFinalChr  <- data.frame()

      for (i in 1:LL){
          filteredScoreFinal     <- data.frame()
          listLen <-  listLen + 1
          mL <- max(length(SampleCovPlus[[listLen]]), length(SampleCovMinus[[listLen]]))
          frames <- floor(mL / N) +1
          for (m in 1:frames){

              if (m != frames){
                 lFrame <-  (((m-1)*N)+1):(m*N)
              }
               if (m == frames){
                  lFrame<- (((m-1)*N)+1):mL
              }
              sample_forward <- as.integer(SampleCovPlus[[listLen]] [  lFrame ])
              sample_reverse <- as.integer(SampleCovMinus[[listLen]][  lFrame ])


              k <- as.integer(sample_forward - sample_reverse)

              #rm(sample_forward,sample_reverse)


              score <- c()
              iPlus  <- which(k>=0)
              iMinus <- which(k<0)
              score[iPlus]  <- pskellam(q=k[iPlus],    lambda1= lambda1[w], lambda2 = lambda2[w], lower.tail = FALSE, log.p = FALSE)
              score[iMinus] <- pskellam(q=k[iMinus]-1, lambda1= lambda1[w], lambda2 = lambda2[w], lower.tail = TRUE, log.p = FALSE)
              positions  <- which(score <= p)


              maxD = width[2]
              minD = width[1]

              filteredScore <- data.frame()
              idx=0

               if (length(positions)>1){
               
                  for (j in 2:length(positions)){
                  
                      A <- positions[j-1]
	                    B <- positions[j]
	                    C <- sign(k[positions[j-1]])
	                    D <- sign(k[positions[j]])

                      if (C != 0  && D != 0 ){
	                       if (B< A+maxD && B> A+minD  && (C == 1  && D== -1)  ){
		                        idx=idx+1
		                        filteredScore[idx,1] <- chrN[i]
		                        filteredScore[idx,2] <- lFrame[positions[j-1]]
		                        filteredScore[idx,3] <- lFrame[positions[j]]
		                        filteredScore[idx,4] <- lFrame[positions[j]]-lFrame[positions[j-1]]
		                          filteredScore[idx,5]  <- sample_forward[positions[j-1]]
                                filteredScore[idx,6] <- sample_reverse[positions[j-1]]
                                
		                        filteredScore[idx,7] <- signif(score[positions[j-1]], digits=30)
		                                    filteredScore[idx,8] <- sample_forward[positions[j]]
                                filteredScore[idx,9] <- sample_reverse[positions[j]]  
                                
		                        filteredScore[idx,10] <- signif(score[positions[j]] , digits=30)
				                     pvalVector <- c(score[positions[j-1]] , score[positions[j]])
				                 filteredScore[idx,11] <- sum(pvalVector)
			                      filteredScore[idx,12] <- -1*log10(filteredScore[idx,7])
                                  
                    
                                   
			                   }
		                  }
                  }
                }
                
              if ( length(filteredScoreFinal) > 0 &&  length(filteredScore) > 0) { filteredScoreFinal <- rbind(filteredScoreFinal,filteredScore)}
              if ( length(filteredScoreFinal) ==0  &&  length(filteredScore) > 0) { filteredScoreFinal <- filteredScore  }
                   


          }  #end frame
          
rm(sample_forward,sample_reverse)          

             if (length(filteredScoreFinalChr) > 0 && length(filteredScoreFinal) > 0 ) {  filteredScoreFinalChr <- rbind(filteredScoreFinalChr, filteredScoreFinal )  }
             if (length(filteredScoreFinalChr) ==0 && length(filteredScoreFinal) > 0 ) {  filteredScoreFinalChr <- filteredScoreFinal   }

     }  #end chromosome


  colnames(filteredScoreFinalChr) <- c("chr","start","end","length","DNaseIcuts.start.forward","DNaseIcuts.start.reverse","pval.forward","DNaseIcuts.end.forward","DNaseIcuts.end.reverse","pval.reverse", "pval.footprint.event","log10.pval.footprint.event")

}   #end sample




if (correction != "none") {

        filteredScoreFinalChr$pval.forward <- p.adjust(filteredScoreFinalChr$pval.forward, method=correction)
        filteredScoreFinalChr$pval.reverse <- p.adjust(filteredScoreFinalChr$pval.reverse, method=correction)
        for (i in 1:length(filteredScoreFinalChr$pval.reverse) ){
           filteredScoreFinalChr$pval.footprint.event[i]          <-  filteredScoreFinalChr$pval.forward[i] + filteredScoreFinalChr$pval.reverse[i]
           filteredScoreFinalChr$log10.pval.footprint.event[i]    <-   -1*log10(filteredScoreFinalChr$pval.footprint.event[i])
        }
        
}

   filteredScoreFinalChr  <-  filteredScoreFinalChr[ which(filteredScoreFinalChr$pval.footprint.event <= p  ) , ]
   
##   #create a GRanges structure
##   repl <- GRanges(seqnames=filteredScoreFinalChr$chr,ranges=IRanges(filteredScoreFinalChr$start, end=filteredScoreFinalChr$end), score=filteredScoreFinalChr$log10.pval.footprint.event)
##   seqlengths(repl)   <- chrL


##return(list( footprints = filteredScoreFinalChr , footprint.events = repl  ))
  return(list( footprint.events = filteredScoreFinalChr  ))


}
